import logging

import typer
from rich.console import Console

from . import python_cli
from .releases import PythonReleases

RICH_CONSOLE = Console()


app = typer.Typer()
logger = logging.getLogger("pirel")


@app.callback(invoke_without_command=True)
def releases_table():
    try:
        py_cli = python_cli.PythonCli()
        logger.info(f"Found interpreter at {py_cli.path!r} (via {py_cli.cmd!r})")
        py_version = py_cli.version
    except FileNotFoundError:
        logger.warning("Could not find an active Python interpreter")
        py_version = None

    releases = PythonReleases()
    RICH_CONSOLE.print("", releases.to_table(py_version))


if __name__ == "__main__":
    app()



import datetime
import json
import urllib.request
from typing import Optional

from rich.table import Table
from rich.text import Text

from . import python_cli

DATE_NOW = datetime.date.today()
DATE_NOW_STR = str(DATE_NOW)


def parse_date(date_str: str) -> datetime.date:
    if len(date_str) == len("yyyy-mm"):
        # We need a full yyyy-mm-dd, so let's approximate
        return datetime.date.fromisoformat(date_str + "-01")
    return datetime.date.fromisoformat(date_str)


def date_style(date: datetime.date) -> str:
    """Returns the style for a date for rich table."""
    if date > DATE_NOW:
        # Future, add italics
        return "italic"
    return ""


class PythonRelease:
    def __init__(self, version: str, data: dict):
        self._version = version
        self._status = data["status"]
        self._released = parse_date(data["first_release"])
        self._end_of_life = parse_date(data["end_of_life"])

    @property
    def version(self) -> str:
        return self._version

    @property
    def status(self) -> Text:
        _status = Text(self._status)
        if _status.plain == "end-of-life":
            _status.stylize("red")
        elif _status.plain == "security":
            _status.stylize("yellow")
        elif _status.plain == "bugfix":
            _status.stylize("green")
        elif _status.plain == "prerelease":
            _status.stylize("blue")
        elif _status.plain == "feature":
            _status.stylize("magenta")
        return _status

    @property
    def released(self):
        _released = Text(str(self._released), style=date_style(self._released))
        return _released

    @property
    def end_of_life(self):
        font_style = date_style(self._end_of_life)
        if self._status == "end-of-life":
            color = "red"
        elif datetime.timedelta(days=30 * 2) + DATE_NOW > self._end_of_life:
            color = "dark_orange"
        elif datetime.timedelta(days=365) + DATE_NOW > self._end_of_life:
            color = "yellow"
        else:
            color = "green"

        eol = Text(str(self._end_of_life), style=f"{font_style} {color}")
        return eol


class PythonReleases:
    def __init__(self) -> None:
        with urllib.request.urlopen(
            "https://raw.githubusercontent.com/python/devguide/refs/heads/main/include/release-cycle.json"
        ) as f:
            self.releases_data = json.load(f)

        self.releases = [
            PythonRelease(version, data) for version, data in self.releases_data.items()
        ]

    def to_table(
        self, active_python_version: Optional[python_cli.PythonVersion] = None
    ) -> Table:
        table = Table(title="Python Releases")

        table.add_column("Version", justify="right", style="cyan", no_wrap=True)
        table.add_column("Status", justify="right", no_wrap=True)
        table.add_column(
            "Released", justify="right", style="bright_black", no_wrap=True
        )
        table.add_column("End-of-life", justify="right", no_wrap=True)

        for release in self.releases:
            row_style = None
            _version = release.version
            if (
                active_python_version
                and active_python_version.as_release == release.version
            ):
                _version = f"* [bold]{release.version}[/bold]"
                row_style = "bold"

            table.add_row(
                _version,
                release.status,
                release.released,
                release.end_of_life,
                style=row_style,
            )

        return table


if __name__ == "__main__":
    from rich.console import Console

    txt = Text("foo", style="italic dark_orange")
    Console().print(txt)



import re
import subprocess
import sys
from dataclasses import dataclass

PYTHON_VERSION_RE = re.compile(r"^Python ([23])\.(\d+)\.(\d+)$")


@dataclass(frozen=True)
class PythonVersion:
    major: int
    minor: int
    patch: int

    @classmethod
    def from_str(cls, version: str) -> "PythonVersion":
        return cls(*map(int, version.split(".")))

    @classmethod
    def this(cls) -> "PythonVersion":
        return cls(*sys.version_info)

    @property
    def as_release(self) -> str:
        return f"{self.major}.{self.minor}"

    def __str__(self) -> str:
        return f"{self.major}.{self.minor}.{self.patch}"

    def __repr__(self) -> str:
        return f"PythonVersion({self.major}.{self.minor}.{self.patch})"


def parse_python_version(python_version_output: str) -> PythonVersion:
    match = PYTHON_VERSION_RE.match(python_version_output)
    if not match:
        raise ValueError(
            f"The Python version output {python_version_output!r} "
            f"does not match the regex {PYTHON_VERSION_RE.pattern!r}"
        )
    major, minor, patch = match.groups()
    return PythonVersion(major, minor, patch)


class PythonCli:
    def __init__(self):
        version_out = None
        for py in ("python", "python3", "python2"):
            try:
                version_out = subprocess.run((py, "--version"), capture_output=True)
                self.version = parse_python_version(version_out.stdout.decode())
                break
            except (FileNotFoundError, ValueError):
                pass

        if version_out is None:
            raise FileNotFoundError("Could not find an active Python interpreter")

        path_out = subprocess.run(
            (py, "-c", "import sys; print(sys.executable)"), capture_output=True
        )

        self.cmd = py
        self.path = path_out.stdout.decode().strip()
